#include "Status.h"
#include "Vram.h"
#include "Print.h"
#include "Chars.h"
#include "Sprite.h"

constexpr byte Char_Remain = Char_Fighter;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus()
{
    PrintS(Vram + VramRowSize * 0 + (WindowWidth + 2) * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 3 + (WindowWidth + 2) * VramStep, "HIGH");
    PrintS(Vram + VramRowSize * 4 + (WindowWidth + 2) * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 7 + (WindowWidth + 2) * VramStep, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 8 + (WindowWidth + 6) * VramStep, CurrentStage + 1);
    PrintRemain();
    PrintScore();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 1 + (WindowWidth + 2) * VramStep, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 5 + (WindowWidth + 2) * VramStep, HiScore);
    PrintC(pVram, '0');
}

void PrintRemain()
{
    ptr<byte> pVram;
    pVram = Vram + VramRowSize * 21 + (WindowWidth + 3) * VramStep;
    if (RemainCount > 1) {
        byte i;
        i = RemainCount - 1;
        if (i > 2) {
            pVram = Put2C(pVram, Char_Remain);
            pVram = Erase2(pVram);
            pVram += VramRowSize - 2 * VramStep;
            pVram = PrintC(pVram, i + '0');
            pVram += 1  * VramStep - VramRowSize;
        }
        else {
            do {
                pVram = Put2C(pVram, Char_Remain);
                --i;
            } while (i > 0);
        }
    }
    if (RemainCount < 3) {
        pVram = Erase2(pVram);
    }
}


void PrintGameOver()
{
    constexpr byte Left = (WindowWidth - 8) / 2;
    PrintS(Vram + VramRowSize * (VramHeight / 2) + Left * VramStep, "GAME OVER");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    {
        static const byte[] TitleBytes = {
            //	G
            0x20, 0x2e, 0x25, 0x2b, 
            0x2c, 0x23, 0x2a, 0x2a, 
            0x24, 0x2b, 0x20, 0x2f, 
            0x20, 0x24, 0x25, 0x25, 
            //	U
            0x2c, 0x23, 0x20, 0x2f, 
            0x2c, 0x23, 0x20, 0x2f, 
            0x2c, 0x23, 0x20, 0x2f, 
            0x20, 0x25, 0x25, 0x21, 
            //	N
            0x2c, 0x2b, 0x20, 0x2f, 
            0x2c, 0x2f, 0x2b, 0x2f, 
            0x2c, 0x23, 0x2d, 0x2f, 
            0x24, 0x21, 0x20, 0x25, 
            //	T
            0x24, 0x2d, 0x27, 0x21, 
            0x20, 0x2c, 0x23, 0x20, 
            0x20, 0x2c, 0x23, 0x20, 
            0x20, 0x24, 0x21, 0x20, 
            //	U
            0x2c, 0x23, 0x20, 0x2f, 
            0x2c, 0x23, 0x20, 0x2f, 
            0x2c, 0x23, 0x20, 0x2f, 
            0x20, 0x25, 0x25, 0x21, 
            //	S
            0x28, 0x27, 0x25, 0x2b, 
            0x24, 0x2b, 0x2a, 0x22, 
            0x28, 0x22, 0x20, 0x2f, 
            0x20, 0x25, 0x25, 0x21, 
        };
        constexpr byte LogoLength = 6;
        constexpr byte LogoWidth = LogoLength * 4;
        constexpr byte LogoX = (32 - LogoWidth) / 2;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 8 + LogoX;
    p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p; ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 4;
        }

        pVram = Vram + VramRowSize * 8 + LogoX - 2;
        repeat (4) {
            pVram[0] = 9;
            pVram[1] = Color_Cyan;
            pVram[2 + LogoWidth] = 8;
            pVram[3 + LogoWidth] = Color_White;
            pVram += VramRowSize;
        }
    }
    PrintS(Vram + VramRowSize * 19 + (WindowWidth - 14) * VramStep / 2, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + (WindowWidth - 14) * VramStep / 2, "OR RETURN KEY");
    PrintS(Vram + VramRowSize * 23 + (WindowWidth - 12) * VramStep, "INUFUTO 2022");
    // {
    //     byte c;
    //     ptr<byte> pVram;
    //     pVram = Vram;
    //     c = 0x20;
    //     repeat (0x60) {
    //         *pVram = c; ++pVram;
    //         ++c;
    //         if ((c & 15) == 0) {
    //             pVram += VramRowSize - 16 * VramStep;
    //         }
    //     }
    // }
}


void AddScore(word pts)
{
    Score += pts;
    if (Score > HiScore) {
        HiScore = Score;
    }
    PrintScore();
}
