#include "Vram.h"
#include "VVram.h"
#include "CopyMemory.h"
#include "Chars.h"

extern const byte[] Chars, Logo;

byte[256] CharColors;

static void SetColor(ptr<byte> pColor, byte color, byte count)
{
    do {
        *pColor = color;
        ++pColor;
        --count;
    } while (count != 0);
}


void InitVram()
{
    static const byte[] ColorSource = {
        Char_EnemyBullet, 1, Color_Yellow,
        Char_Bang, 4, Color_Red,
        Char_Item, 4, Color_Green,
        Char_EnemyRowA, 4, Color_Magenta,
        Char_EnemyRowB, 4, Color_Red,
        Char_EnemyRowC, 4, Color_Yellow,
        0xff
    };
    ptr<byte> pColor, pSource;

    pColor = CharColors;
    repeat (0) {
        *pColor = Color_White;
        ++pColor;
    }

    pSource = ColorSource;
    while (pSource[0] != 0xff) {
        byte count, color;
        pColor = CharColors + pSource[0];
        count = pSource[1];
        color = pSource[2];
        pSource += 3;
        do {
            *pColor = color;
            ++pColor;
            --count;
        } while (count > 0);
    }
    CopyPattern(Char_Ascii, Pattern_Ascii, Pattern_EnemyB - Pattern_Ascii);
    CopyPattern(Char_Fighter, Pattern_Fighter, Pattern_Logo - Pattern_Fighter);
    CopyPattern(Char_EnemyRowA, Pattern_EnemyA, 4);
    CopyPattern(Char_EnemyRowB, Pattern_EnemyB, 4);
    CopyPattern(Char_EnemyRowC, Pattern_EnemyC, 4);
    CopyPattern(Char_Logo + 0x80, Pattern_Logo, 16);
}


void ClearScreen()
{
    FillMemory(VramBase, VramRowSize * (VramHeight + MarginHeight), Color_White);
}


ptr<byte> Put2C(ptr<byte> pVram, byte c)
{
    repeat (2) {
        repeat (2) {
            *pVram = c; ++pVram;
            ++c;
        }
        pVram += VramRowSize - 2 * VramStep;
    }
    return pVram + 2 * VramStep - VramRowSize * 2;
}


ptr<byte> Erase2(ptr<byte> pVram)
{
    repeat (2) {
        repeat (2) {
            *pVram = Char_Space; ++pVram;
        }
        pVram += VramRowSize - 2 * VramStep;
    }
    return pVram + 2 * VramStep - VramRowSize * 2;
}
